#!/bin/sh
#----------------------------------------------------------------------------
# /var/install/config.d/nginx.sh - configuration generator script for nginx
#
# Copyright (c) 2001-2021 Ansgar Puester <ansgar.puester@freenet.de>
#
# Creation:     18.05.2014  ap
# Last Update:  $Id: nginx.sh 83702 2021-02-17 12:46:14Z ansgar $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------

# include eislib
. /var/install/include/eislib

pgmname=$0

# set variables
basefile=/etc/config.d/base
nginxfile=/etc/config.d/nginx
nginx_config_file=/etc/nginx/nginx.conf


ngx_log_root='/var/log/nginx/'


#-------------------------------------------------------------------------------
# global functions
#-------------------------------------------------------------------------------

# isnum
# =====
#
# check argument if it is numerical
#
# Call  :  isnum <argument>
#
# Return:  0  -> argument is numerical
#          1  -> argument is not numerical

isnum ()
{
  echo "$1" | grep '^[0-9][0-9]*$' > /dev/null 2>&1
  return $?
}

#-------------------------------------------------------------------------------
# check nginx_config file
#-------------------------------------------------------------------------------
check_nginx_config ()
{

# check NGINX_LB_SERVER_x_BACKUP
if [ "$NGINX_SERVER_TYPE" = 'load_balancer' ]
then
   count_backup=0
   idx=1
   while [ "$idx" -le "$NGINX_LB_SERVER_N" ]
   do
      eval backup='$NGINX_LB_SERVER_'${idx}'_BACKUP'
      if [ "$backup" = 'yes' ]
      then
         count_backup=`expr $count_backup + 1`
      fi
      idx=`expr $idx + 1`
   done
fi

if [ "$count_backup" = "$NGINX_LB_SERVER_N" ]
then
   mecho --error "All load balancer server are defined as backup. That's impossible."
   anykey
fi

}

#-------------------------------------------------------------------------------
# create nginx logrotate configuration
#-------------------------------------------------------------------------------
create_nginx_logrotate_conf()
{

# check NGINX_ROTATE_LOGS
if [ "$NGINX_ROTATE_LOGS" = 'no' ]
then
   rm -f /etc/logrotate.d/nginx
else
   # NGINX_ACCESS_LOG='off'
   if [ "$NGINX_ACCESS_LOG" = 'off' ]
   then
      NGINX_ACCESS_LOG=''
   fi

   # NGINX_ACCESS_LOG starts with syslog:server=
   access_start=$(echo $NGINX_ACCESS_LOG | cut -c 1-14)
   if [ "$access_start" = 'syslog:server=' ]
   then
      NGINX_ACCESS_LOG=''
   fi

   # NGINX_ERROR_LOG  starts with syslog:server=
   error_start=$(echo $NGINX_ERROR_LOG | cut -c 1-14)
   if [ "$error_start" = 'syslog:server=' ]
   then
      NGINX_ERROR_LOG=''
   fi

   # Check if absolute filenames or relative filenames
   access_start=$(echo $NGINX_ACCESS_LOG | cut -c 1)
   if [[ ! "$access_start" = '/' && ! -z "$access_start" ]]
   then
      NGINX_ACCESS_LOG="$ngx_log_root$NGINX_ACCESS_LOG"
   fi
   error_start=$(echo $NGINX_ERROR_LOG | cut -c 1)
   if [[ ! "$error_start" = '/' && ! -z "$error_start" ]]
   then
      NGINX_ERROR_LOG="$ngx_log_root$NGINX_ERROR_LOG"
   fi

   flist=''
   # add filename an trailing space
   [ -n "$NGINX_ACCESS_LOG" ] && flist="${flist}${NGINX_ACCESS_LOG} "
   [ -n "$NGINX_ERROR_LOG" ] && flist="${flist}${NGINX_ERROR_LOG} "

   if [ ! "$NGINX_ACCESS_LOG" = '' -o ! "$NGINX_ERROR_LOG" = '' ]
   then
   {
       echo '#---------------------------------------------------------------------'
       echo "# /etc/logrotate.d/nginx file generated by $0"
       echo '#'
       echo '# Do not edit this file, edit /etc/config.d/nginx'
       echo "# Creation Date: `date`"
       echo '#---------------------------------------------------------------------'
       #echo "$NGINX_ACCESS_LOG $NGINX_ERROR_LOG {"
       echo "$flist{"
       echo "    $NGINX_LOGROTATE_INTERVAL"
       echo '    missingok'
       echo "    rotate $NGINX_LOGROTATE_COUNT"
       echo '    compress'
       #echo '    delaycompress'
       echo '    notifempty'
       echo '    create 644 root root'
       echo '    sharedscripts'
       echo '    postrotate'
       echo '        test -f /var/run/nginx.pid && /sbin/killproc -p /var/run/nginx.pid -USR1 /sbin/nginx || true'
       echo '    endscript'
       echo '}'
   } > /etc/logrotate.d/nginx
   fi

   if [ "$NGINX_ACCESS_LOG" = '' -a "$NGINX_ERROR_LOG" = '' ]
   then
      rm -f /etc/logrotate.d/nginx
   fi

fi

}

#-------------------------------------------------------------------------------
# create nginx_config file
#-------------------------------------------------------------------------------
create_nginx_config ()
{
if [ "$START_NGINX" = "yes" ]
then
   if [ "$NGINX_MAN_CONFIG" != 'yes' ]; then
        mecho --std "Creating nginx configuration ..."

    {
        # Check if absolute filenames or relative filenames
        access_start=$(echo $NGINX_ACCESS_LOG | cut -c 1)
        if [ ! "$access_start" = '/' ]
        then
           access_start=$(echo $NGINX_ACCESS_LOG | cut -c 1-14)
           if [[ ! "$access_start" = 'syslog:server=' && ! $NGINX_ACCESS_LOG = 'off' ]]
           then
              NGINX_ACCESS_LOG="$ngx_log_root$NGINX_ACCESS_LOG"
           fi
        fi
        error_start=$(echo $NGINX_ERROR_LOG | cut -c 1)
        if [ ! "$error_start" = '/' ]
        then
           error_start=$(echo $NGINX_ERROR_LOG | cut -c 1-14)
           if [ ! "$error_start" = 'syslog:server=' ]
           then
              NGINX_ERROR_LOG="$ngx_log_root$NGINX_ERROR_LOG"
           fi
        fi

        echo "#-------------------------------------------------------------------------------"
        echo "# nginx_config file generated by $pgmname"
        echo "#"
        echo "# Do not edit this file, edit $nginxfile"
        echo "# Creation date: `date`"
        echo "#-------------------------------------------------------------------------------"
        echo
        echo
        echo '#user  nobody;'
        echo "worker_processes  $NGINX_WORKER_PROCESSES;"
        echo
        echo '#error_log  ${ngx_log_root}error.log;'
        echo '#error_log  ${ngx_log_root}error.log  notice;'
        echo '#error_log  ${ngx_log_root}error.log  info;'
        echo "error_log  $NGINX_ERROR_LOG;"
        echo
        echo '#pid        /var/run/nginx.pid;'
        echo
        echo
        echo 'events {'
        echo "	  worker_connections  $NGINX_WORKER_CONNECTIONS;"
        echo '}'
        echo
        echo
        echo 'http {'
        echo '    include       mime.types;'
        echo '    default_type  application/octet-stream;'
        echo
        echo "    #log_format  main  '\$remote_addr - \$remote_user [\$time_local] \"\$request\" ' "
        echo "    #                  '\$status \$body_bytes_sent \"\$http_referer\" ' "
        echo "    #                  '\"\$http_user_agent\" \"\$http_x_forwarded_for\" ';"
        echo
        LOG_FORMAT=''
   if [ "$NGINX_LOGANON" = 'yes' ]; then
        echo "    # Use anonymized IP logging"
        echo "    include anonymized_${NGINX_LOGANON_MODE}.conf;"
        echo
        LOG_FORMAT=' anonymized'
   fi
       #echo '    #access_log  ${ngx_log_root}access.log  main;'
        echo "    access_log  $NGINX_ACCESS_LOG$LOG_FORMAT;"
        echo
        echo '    sendfile        on;'
        echo '    #tcp_nopush     on;'
        echo
        echo '    #keepalive_timeout  0;'
        echo '    keepalive_timeout  65;'
        echo
        echo '    #gzip  on;'
   case "$NGINX_SERVER_TYPE" in
   load_balancer)
        echo
        echo "    upstream $NGINX_UPSTREAM_NAME {"
     idx=1
     while [ "$idx" -le "$NGINX_LB_SERVER_N" ]
     do
        opt=''
        eval address='$NGINX_LB_SERVER_'${idx}'_ADDRESS'
        eval weight='$NGINX_LB_SERVER_'${idx}'_WEIGHT'
        [ "$weight" = '' ] || opt="$opt weight=$weight"
        eval max_fails='$NGINX_LB_SERVER_'${idx}'_MAX_FAILS'
        [ "$max_fails" = '' ] || opt="$opt max_fails=$max_fails"
        eval fail_timeout='$NGINX_LB_SERVER_'${idx}'_FAIL_TIMEOUT'
        [ "$fail_timeout" = '' ] || opt="$opt fail_timeout=$fail_timeout"
        eval backup='$NGINX_LB_SERVER_'${idx}'_BACKUP'
        [ "$backup" = 'yes' ] && opt="$opt backup"
        echo "       server $address$opt;"

        idx=`expr $idx + 1`
     done
        echo '    }'

   esac
        echo
        echo '    server {'
        idx=1
        while [ "${idx}" -le "${NGINX_SERVER_LISTEN_N}" ]
        do
            eval listen='$NGINX_SERVER_LISTEN_'${idx}
            echo "        listen       $listen;"
            let idx=$idx+1
        done
        echo
        echo "        server_name  $NGINX_SERVER_NAME;"
        echo
        echo '        #charset koi8-r;'
        echo
        echo '        #access_log  ${ngx_log_root}host.access.log  main;'
        echo

   if [ "$NGINX_STUB_STATUS_LOAD" = 'yes' ] 
   then
	echo '        #status modul'
	echo "        location "$NGINX_STUB_STATUS_LOCATION" {"
	echo '            stub_status;'
	echo '        }'
	echo '        stub_status on;'
        echo
   fi

   case "$NGINX_SERVER_TYPE" in
   web_server)
        echo '        #web_server'
        echo '        location / {'
        echo '            /var/lib/nginx/htdocs/;'
        echo '            index  index.html index.htm;'
        echo '        }'
        ;;
   reverse_proxy)
        echo '        #reverse_proxy'
        echo '        location / {'
        echo "            proxy_buffering   $NGINX_PROXY_BUFFERING;"
        echo "            proxy_pass        $NGINX_PROXY_PASS_URL;"
        echo '        }'
        ;;
   load_balancer)
        echo '        #load_balancer'
        echo '        location / {'
        echo "            proxy_pass        http://$NGINX_UPSTREAM_NAME;"
        echo '        }'
        ;;
   *)
        echo '        #error'
        ;;
   esac

        echo
        echo '        #error_page  404              /404.html;'
        echo
        echo '        # redirect server error pages to the static page /50x.html'
        echo '        #'
        echo '        error_page   500 502 503 504  /50x.html;'
        echo '        location = /50x.html {'
        echo '            root   /var/lib/nginx/htdocs/;'
        echo '        }'
        echo
        echo '        # proxy the PHP scripts to Apache listening on 127.0.0.1:80'
        echo '        #'
        echo '        #location ~ \.php$ {'
        echo '        #    proxy_pass   http://127.0.0.1;'
        echo '        #}'
        echo
        echo '        # pass the PHP scripts to FastCGI server listening on 127.0.0.1:9000'
        echo '        #'
        echo '        #location ~ \.php$ {'
        echo '        #    root           /var/lib/nginx/htdocs/;'
        echo '        #    fastcgi_pass   127.0.0.1:9000;'
        echo '        #    fastcgi_index  index.php;'
        echo '        #    fastcgi_param  SCRIPT_FILENAME  /scripts$fastcgi_script_name;'
        echo '        #    include        fastcgi_params;'
        echo '        #}'
        echo
        echo "        # deny access to .htaccess files, if Apache's document root"
        echo "        # concurs with nginx's one"
        echo '        #'
        echo '        #location ~ /\.ht {'
        echo '        #    deny  all;'
        echo '        #}'
        echo '    }'
        echo
        echo
        echo '    # another virtual host using mix of IP-, name-, and port-based configuration'
        echo '    #'
        echo '    #server {'
        echo '    #    listen       8000;'
        echo '    #    listen       somename:8080;'
        echo '    #    server_name  somename  alias  another.alias;'
        echo
        echo '    #    location / {'
        echo '    #        root   /var/lib/nginx/htdocs/;'
        echo '    #        index  index.html index.htm;'
        echo '    #    }'
        echo '    #}'
        echo
        echo
        echo '    # HTTPS server'
        echo '    #'
        echo '    #server {'
        echo '    #    listen       443 ssl;'
        echo '    #    server_name  localhost;'
        echo
        echo '    #    ssl_certificate      cert.pem;'
        echo '    #    ssl_certificate_key  cert.key;'
        echo
        echo '    #    ssl_session_cache    shared:SSL:1m;'
        echo '    #    ssl_session_timeout  5m;'
        echo
        echo '    #    ssl_ciphers  HIGH:!aNULL:!MD5;'
        echo '    #    ssl_prefer_server_ciphers  on;'
        echo
        echo '    #    location / {'
        echo '    #        root   /var/lib/nginx/htdocs/;'
        echo '    #        index  index.html index.htm;'
        echo '    #    }'
        echo '    #}'
        echo
        echo '}'

    } > $nginx_config_file
   else
      mecho --warn "Attention: NGINX_MAN_CONFIG='yes' nginx configuration not changed"
   fi
fi
}

#===============================================================================
# main
#===============================================================================

# read configuration
. $nginxfile
. $basefile

#mecho --info "nginx.sh Stopping service"
#sh /etc/init.d/nginx stop

#check_nginx_config

create_nginx_config

create_nginx_logrotate_conf

#mecho --info "nginx.sh Starting service"
#sh /etc/init.d/nginx start
mecho --info "Finished"

#===============================================================================
